----------------------------------------------------------------------------------------------------------
--
--  eTargetList
--
-- written by Elgor
-- released under the Creative Commons License By-Nc-Sa: http://creativecommons.org/licenses/by-nc-sa/3.0/
--
-- release notes
-- ver. 0.1.0 : initial release(alpha-state)
-- ver. 0.1.1 : fixed scaling bug
--			  : added GUI config
-- ver. 0.1.2 : fixed bug with auto-resetting config
-- ver. 0.2.0 : fixed bug with bars are not shown correctly when players/NPCs health bars are shown
--			  : added some config options
--			  : removed lock option
--			  : added some macro I/F
--			  : added Addon Manager support
----------------------------------------------------------------------------------------------------------
--========================================================================================================
-- Addon namespace definition
--========================================================================================================
local eTargetList = {};
_G.eTargetList = eTargetList;

--========================================================================================================
-- Slash command
--========================================================================================================
SLASH_ETL1 = "/etl";
SlashCmdList["ETL"] = function(editBox, msg)
	eTargetList.CmdHdr(msg);
end

--========================================================================================================
-- Global vars
--========================================================================================================
eTL_Config_Save = {};

--========================================================================================================
-- Local vars
--========================================================================================================
----------
-- Configs
----------
local onConfig = false;
local tempConfig = {};
local defaultConfig = {
	["version"] =		013,
	["init"] =			false,
	["enable"] =		true,
	["pos_x"] =			300,
	["pos_y"] = 		200,
	["scale"] =			1,
	["fill_order"] =	"topdown",	-- or bottomup
	["bar_tex"] =		"smooth",	-- or flat or banto or round
	["bar_h"] =			24,
	["bar_w"] =			200,
	["bar_s"] =			33,
	["show_lvl"] =		true,
	["show_hp"] =		true,
};
local scalestep = {
	[1]  = "0.2",
	[2]  = "0.4",
	[3]  = "0.6",
	[4]  = "0.8",
	[5]  = "1.0",
	[6]  = "1.2",
	[7]  = "1.4",
	[8]  = "1.6",
	[9]  = "1.8",
	[10] = "2.0",
};
local barheightstep = {
	[1] =	"12",
	[2] =	"16",
	[3] =	"20",
	[4] =	"24",
	[5] =	"28",
	[6] =	"32",
	[7] =	"36",
};
local barwidthstep = {
	[1] =	"125",
	[2] =	"150",
	[3] =	"175",
	[4] =	"200",
	[5] =	"225",
	[6] =	"250",
	[7] =	"275",
};
local barspacestep = {
	[1] =	"0",
	[2] =	"16",
	[3] =	"33",
	[4] =	"50",
	[5] =	"66",
	[6] =	"100",
};

----------------
-- Internal vars
----------------
local bbindex = {
	[1] = 0,
	[2] = 0,
	[3] = 0,
	[4] = 0,
	[5] = 0,
	[6] = 0,
	[7] = 0,
	[8] = 0,
};
local init_done = false;

----------
-- Strings
----------
local AddonName = 			"eTargetList";
local AddonVer =			"v0.2.0";
local SmoothBar =			"interface/AddOns/eTargetList/smooth.tga";
local FlatBar =				"interface/AddOns/eTargetList/flat.tga";
local BantoBar =			"interface/AddOns/eTargetList/bantobar.tga";
local RoundBar =			"interface/AddOns/eTargetList/round.tga";
local TgtTexturePath =		"interface/AddOns/eTargetList/round32x32.tga";
local DescMoveButton =		"|cff00CCFFYou can move this frame by mouse dragging|r";
local DescConfigButton =	"|cffFFFF00Config Menu|r";
local DescAM =				"eTargetList is an addon that displays a list of hostile units around you as HP bars."

--------------------------------------------
-- Local references to API for faster access
--------------------------------------------
local _G = 					_G;
local OBB_ChangeTraget =	OBB_ChangeTraget;
local UnitExists =			UnitExists;
local UnitIsDeadOrGhost =	UnitIsDeadOrGhost;
local UnitCanAttack =		UnitCanAttack;
local TargetNearestEnemy =	TargetNearestEnemy;
local UseSkill = 			UseSkill;
local pairs = 				pairs;
local GetUIScale =			GetUIScale;
local tonumber =			tonumber;
local math_floor =			math.floor;

--========================================================================================================
-- Functions
--========================================================================================================
--------
-- Print
--------
local function DCFmsg(str, r, g, b)
	local red, green, blue;
	if( r == nil or g == nil or b == nil ) then
		red = 1;
		green = 1;
		blue = 0;
	else
		red = r;
		green = g;
		blue = b;
	end
	DEFAULT_CHAT_FRAME:AddMessage(str, red, green, blue);
end

-----------------
-- OnLoad Handler
-----------------
function eTargetList.OnLoad(this)
	this:RegisterEvent("VARIABLES_LOADED");
	this.lastUpdate = 0;
	this.count = 0;
end

----------------
-- Event Handler
----------------
function eTargetList.OnEvent(this, event)
	if( event == "VARIABLES_LOADED" ) then
		eTargetList.LoadConfig();
		if( tempConfig.version < 11 ) then		-- not compatible config data
			eTargetList.ResetConfig();
			eTargetList.LoadConfig();			-- reload config
			onConfig = true;
		end
		eTargetList.InitFrame();
		eTargetList.SaveConfig();
		init_done = true;
		if( AddonManager ) then
			local addon = {
				name = 			AddonName,
				version =		AddonVer,
				author =		"Elgor",
				description =	DescAM,
--				icon =			nil,
				category =		"Other",
--				configFrame =	nil,
				slashCommands =	SLASH_ETL1,	-- not slashCommand, needs to be XXXs
--				miniButton =	nil,
				onClickScript =	eTargetList.AMOnClick,
--				disableScript = nil,
--				enableScript =	nil,
			};
			if( AddonManager.RegisterAddonTable ) then
				AddonManager.RegisterAddonTable(addon);
			else
				AddonManager.RegisterAddon(addon.name, addon.description, addon.icon, addon.category,
					addon.configFrame, addon.slashCommand, addon.miniButton, addon.version, addon.author);
			end
		else
			local name = 	"|cffFFFF00"..AddonName.."|r";
			local ver =		"|cffFFFF00"..AddonVer.."|r";
			local slash =	"|cffFFFF00"..SLASH_ETL1.."|r";
			DCFmsg("["..name.." "..ver.."] loaded. Use "..slash.." for Configuration.", 0, 1, 0);
		end
	end
end

--------------------
-- Initialize frames
--------------------
function eTargetList.InitFrame()
	-- move frame
	local scale = GetUIScale();
	local frame = _G["eTL_MoveFrame"];
	local sframe = _G["eTL_MoveFrameButton"];
	frame:ClearAllAnchors();
	frame:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", tempConfig.pos_x/scale, tempConfig.pos_y/scale);
	if( onConfig ) then
		frame:SetMouseEnable(true);
		sframe:Show();
	else
		frame:SetMouseEnable(false);
		sframe:Hide();
	end
	-- main frame
	frame =			_G["eTL_Frame"];
	sframe =		_G["eTL_FrameTitle"];
	local iframe =	_G["eTL_FrameInfo"];
	local yOffset = 0;
	if( tempConfig.bar_s == 0 ) then
		yOffset = tempConfig.bar_h;
	else
		yOffset = tempConfig.bar_h * (1 + (tempConfig.bar_s / 100));
	end
	local w = 10 + tempConfig.bar_w + 10 + tempConfig.bar_h + 10;
	local h = 10 + 20 + 10 + (#bbindex * (yOffset + 1)) + 10;
	frame:SetSize(w,h);
	sframe:SetSize(w,20);
	iframe:SetSize(w,20);
	frame:ClearAllAnchors();
	frame:SetAnchor("TOPLEFT", "TOPLEFT", "eTL_MoveFrame", 0, 0);
	frame:SetScale(tempConfig.scale);
	frame:SetMouseEnable(false);
	frame:SetBackdropTileAlpha(0);
	if( onConfig ) then
		frame:SetBackdropEdgeAlpha(1);
		sframe:SetText(AddonName);
		iframe:SetText(AddonVer.."   Created by Elgor");
	else
		frame:SetBackdropEdgeAlpha(0);
		sframe:SetText("");
		iframe:SetText("");
	end
	-- Config button
	frame = _G["eTL_FrameConfigButton"];
	if( onConfig ) then
		frame:SetScale(1/tempConfig.scale);		-- keep original scale
		frame:Show();
	else
		frame:Hide();
	end
	-- Bars
	local barTexPath = "";
	local hlTexPath = "";
	if( tempConfig.bar_tex == "smooth" ) then
		barTexPath = SmoothBar;
		hlTexPath  = FlatBar;
	elseif( tempConfig.bar_tex == "flat") then
		barTexPath = FlatBar;
		hlTexPath  = FlatBar;
	elseif( tempConfig.bar_tex == "banto") then
		barTexPath = BantoBar;
		hlTexPath  = FlatBar;
	elseif( tempConfig.bar_tex == "round") then
		barTexPath = RoundBar;
		hlTexPath  = RoundBar;
	else
		barTexPath = SmoothBar;
		hlTexPath  = FlatBar;
	end
	local lvl = "";
	if( tempConfig.show_lvl ) then
		lvl = "lvl";
	end
	local hp = "";
	if( tempConfig.show_hp ) then
		hp = "(%HP)";
	end
	local hlw = tempConfig.bar_w + (tempConfig.bar_h * 0.2);
	local hlh = tempConfig.bar_h + (tempConfig.bar_h * 0.2);
	for i = 1, #bbindex do
		local pFrame =	_G["eTL_Frame"..i];
		local bgFrame =	_G["eTL_Frame"..i.."BG"];
		local hFrame =	_G["eTL_Frame"..i.."HL"];
		local bFrame =	_G["eTL_Frame"..i.."Bar"];
		local tFrame =	_G["eTL_Frame"..i.."Name"];
		local aFrame =	_G["eTL_Frame"..i.."TGT"];
		local parent = "";
		local x = 0;
		local y = 0;
		-- bar parent frames
		pFrame:RegisterForClicks("LeftButton", "RightButton");
		pFrame.bbID = 0;
		pFrame.hp = 0;
		pFrame.IsTgt = false;
		if( i == 1 ) then
			parent = pFrame:GetParent():GetName();
			x = 10;
			y = tempConfig.bar_h;
		else
			parent = _G["eTL_Frame"..(i-1)]:GetName();
			x = 0;
			y = yOffset;
		end
		pFrame:ClearAllAnchors();
		pFrame:SetAnchor("TOPLEFT", "TOPLEFT", parent, x, y);
		if( onConfig ) then
			pFrame:Show();
		else
			pFrame:Hide();
		end
		-- bar background frames
		bgFrame:SetTexture(barTexPath);
		bgFrame:SetColor(0,0,0);	-- BG color(black)
		bgFrame:SetSize(tempConfig.bar_w, tempConfig.bar_h);
		-- target highlight
		hFrame:SetSize(hlw,hlh);
		hFrame:SetTexture(hlTexPath);
		hFrame:SetColor(0,0,0);
		parent = bgFrame:GetName();
		hFrame:ClearAllAnchors();
		hFrame:SetAnchor("CENTER", "CENTER", parent, 0, 0);
		if( onConfig ) then
			hFrame:Show();
		else
			hFrame:Hide();
		end
		-- bar frames
		bFrame:SetTexture(barTexPath);
		bFrame:SetColor(1,1,1);	-- OnConfig color(white)
		bFrame:SetSize(tempConfig.bar_w, tempConfig.bar_h);
		if( onConfig ) then
			bFrame:Show();
		else
			bFrame:Hide();
		end
		-- bar text frame
		tFrame:SetText("");			-- need to refresh
		local fSize = math_floor(tempConfig.bar_h/2);
		if( fSize < 8 ) then
			fSize = 8;
		end
		tFrame:SetFontSize(fSize);
		tFrame:SetSize(tempConfig.bar_w, tempConfig.bar_h);
		if( onConfig ) then
			tFrame:Show();
		else
			tFrame:Hide();
		end
		local index = 0;
		if( tempConfig.fill_order == "topdown" ) then
			index = i;
		else
			index = #bbindex - (i - 1);
		end
		tFrame:SetText(lvl.." Name"..tostring(index)..hp);
		-- aggro frame
		aFrame:SetTexture(TgtTexturePath);
		aFrame:SetColor(1,1,1);
		aFrame:SetSize(tempConfig.bar_h, tempConfig.bar_h);	--square size
		if( onConfig ) then
			aFrame:Show();
		else
			aFrame:Hide();
		end
	end
end

--------------------------
-- Mouse drag event(start)
--------------------------
function eTargetList.MoveFrameOnMouseDown(this)
	this:GetParent():StartMoving("TOPLEFT");
end

------------------------
-- Mouse drag event(end)
------------------------
function eTargetList.MoveFrameOnMouseUp(this)
	this:GetParent():StopMovingOrSizing();
	tempConfig.pos_x, tempConfig.pos_y = this:GetParent():GetPos();
	eTargetList.SaveConfig();
end

--------------------------
-- Mouse over event(start)
--------------------------
function eTargetList.MoveFrameOnEnter(this)
	local frame = _G["GameTooltip"];
	frame:ClearAllAnchors();
	frame:SetOwner(this, "ANCHOR_LEFT", 0, 0);
	frame:ClearLines();
	frame:SetText(DescMoveButton);
	frame:Show();
end

--------------------------
-- Mouse over event(end)
--------------------------
function eTargetList.MoveFrameOnLeave(this)
	_G["GameTooltip"]:Hide();
end

------------------
-- Bar click event
------------------
function eTargetList.BarOnClick(this, key)
	if( this.bbID ~= 0 ) then
		OBB_ChangeTraget(this.bbID);
		if( key == "RBUTTON" ) then
			UseSkill(1, 1); -- start auto-attack
		end
	end
end

-------------------
-- OnUpdate handler
-------------------
function eTargetList.OnUpdate(this)
	if( onConfig or not tempConfig.enable or not init_done ) then
		return;
	end

	if( (GetTime() - this.lastUpdate) < 0.2 ) then
		return;
	else
		this.lastUpdate = GetTime();
	end

	this.count = 0;
	for i = 1, #bbindex do
		bbindex[i] = 0;
	end

	-- create bloodbar index table from system frame
	for i = 1, 40 do
		local barFrame = _G["OBloodBar"..i];
		local r, g, b =	_G[barFrame:GetName().."Title"]:GetColor();
		-- filter by color, r value : yellow mob = 1, red mob = 0.929, npc = 0, player = 0.7
		if( barFrame:IsVisible() and r >= 0.9 ) then
			if( this.count < #bbindex ) then
				this.count = this.count + 1;
				if( tempConfig.fill_order == "topdown" ) then
					bbindex[this.count] = i;
				else
					if( this.count > 0 ) then
						bbindex[#bbindex - (this.count - 1)] = i;
					else
						bbindex[#bbindex] = i;
					end
				end
			else
				break;
			end
		end
	end

	local index = 0;
	local stopper = 0;
	local counter = 0;
	local step = 0
	if( tempConfig.fill_order == "topdown" ) then
		index = 1;
		stopper = #bbindex;
		step = 1;
		counter = this.count;
	else
		index = #bbindex;
		stopper = 1;
		step = -1;
		if( this.count > 0 ) then
			counter = #bbindex - (this.count - 1);
		else
			counter = #bbindex + 1;
		end
	end

	-- fill bloodbar data to addon frame
	for i = index, stopper, step do
		if( tempConfig.fill_order == "topdown" ) then
			if( i <= counter ) then
				eTargetList.ShowBars(this, i);
			else
				eTargetList.HideBars(this, i);
			end
		else
			if( i >= counter ) then
				eTargetList.ShowBars(this, i);
			else
				eTargetList.HideBars(this, i);
			end
		end
	end

end

------------
-- Hide bars
------------
function eTargetList.HideBars(this, barIndex)
	local i = barIndex;
	local frame = _G[this:GetName()..i];
	frame.IsTgt = false;
	frame.bbID = 0;
	frame.hp = 0
	frame:Hide();
end

------------
-- Show bars
------------
function eTargetList.ShowBars(this, barIndex)
	local i = 				barIndex;
	local barFrame = 		_G["OBloodBar"..bbindex[i]];
	local aFrame = 			_G[barFrame:GetName().."BarTargetU"];
	local uFrame = 			_G[barFrame:GetName().."BarUTarget"];
	local name = 			_G[barFrame:GetName().."Title"]:GetText();
	local r, g, b = 		_G[barFrame:GetName().."Title"]:GetColor();
	local lvl =				_G[barFrame:GetName().."BarLevel"]:GetText();
	local hp =				_G[barFrame:GetName().."BarValue"]:GetText();
	-- bar
	local frame = _G[this:GetName()..i.."Bar"];
	local w, h = _G[this:GetName()..i.."BG"]:GetSize();
	w = w * (hp/100);
	frame:SetSize(w, h);
	if( tempConfig.bar_tex == "flat" ) then
		if( r == 1 ) then
			frame:SetColor(1, 0.85, 0);
		else
			frame:SetColor(0.8, 0, 0);
		end
	else
		frame:SetColor(r, g, b);
	end
	frame:Show();
	-- bar text
	frame = _G[this:GetName()..i.."Name"];
	local nameText = name;
	if( tempConfig.show_lvl ) then
		nameText = lvl.." "..nameText;
	end
	if( tempConfig.show_hp ) then
		nameText = nameText.."("..hp.."%)";
	end
	frame:SetText(nameText);
	frame:Show();
	-- aggro indicator
	frame = _G[this:GetName()..i.."TGT"];
	if( aFrame:IsVisible() ) then
		frame:Show();
		frame:SetColor(1, 0, 0);
	else
		frame:Hide();
	end
	-- target high-light
	frame = _G[this:GetName()..i];
	local hlFrame = _G["eTL_Frame"..i.."HL"];
	if( uFrame:IsVisible() ) then
		if( r == 1 ) then				-- yellow mob
			hlFrame:SetColor(0.1, 1, 1)	-- pale blue
		else							-- others
			hlFrame:SetColor(1, 1, 0.4)	-- pale yellow
		end	
		hlFrame:Show();
		frame:Show();
		frame.IsTgt = true;
	else
		hlFrame:Hide();
		frame.IsTgt = false;
	end
	frame.bbID = barFrame:GetID();
	frame.hp = tonumber(hp);
	frame:Show();
end

--========================================================================================================
-- Macro I/F
--========================================================================================================
-------------------
-- Target lowest HP
-------------------
function TgtLowestHP()
	local aggro = false;
	eTargetList.TargetLowestHP(aggro);
end

------------------------------
-- Target lowest HP with aggro
------------------------------
function TgtLowestHPAggro()
	local aggro = true;
	eTargetList.TargetLowestHP(aggro);
end

----------------------------------------------------------
-- Target non-aggro(keeps current target if there is none)
----------------------------------------------------------
function TgtNoneAggro()
	for i = 1, #bbindex do
		local frame = _G["eTL_Frame"..i];
		if( not eTargetList.IsAggro(frame) and frame.bbID ~= 0 ) then
			OBB_ChangeTraget(frame.bbID);
			return;
		end
	end
end

----------------------------------------------------------------------
-- Target random(keeps current target if there is only one targetable)
----------------------------------------------------------------------
function TgtAtRandom()
	if( not UnitExists("target") or 
		UnitIsDeadOrGhost("target") or 
		not UnitCanAttack("player", "target") ) then
		TargetNearestEnemy();
		if( UnitExists("target") and not UnitIsDeadOrGhost("target") ) then
			return;
		end
	end
	local curridx = 0;
	local tgtidx = 0;
	local nextidx = 0;
	for i = 1, #bbindex do
		if( bbindex[i] ~= 0 ) then
			curridx = i;
			if( _G["eTL_Frame"..i].IsTgt ) then
				tgtidx = i;
				if( tempConfig.fill_order == "topdown" ) then
					if( i == #bbindex ) then
						nextidx = 1;
					else
						nextidx = i + 1;
						if( bbindex[nextidx] == 0 ) then
							nextidx = 1;
						end
					end
				else
					if( i == 1 ) then
						nextidx = #bbindex;
					else
						nextidx = i - 1;
					end
				end
			end
		end
	end
	local bbid = 0;
	if( tgtidx ~= 0 and nextidx ~= 0 ) then
		bbid = _G["eTL_Frame"..nextidx].bbID;
		if( bbid == 0 ) then
			bbid = _G["eTL_Frame"..curridx].bbID;
		end
	elseif( curridx ~= 0 ) then
		bbid = _G["eTL_Frame"..curridx].bbID;
	end
	OBB_ChangeTraget(bbid);
end

-------------------
-- Target lowest HP
-------------------
function eTargetList.TargetLowestHP(aggro)
	local minHP = 100;
	local bbid = 0;
	for i = 1, #bbindex do
		local frame = _G["eTL_Frame"..i];
		local hp = frame.hp;
		if( hp > 0 and hp <= minHP and frame:IsVisible() ) then
			if( aggro ) then
				if( eTargetList.IsAggro(frame) ) then
					minHP = hp;
					bbid = frame.bbID;
				end
			else
				minHP = hp;
				bbid = frame.bbID;
			end
		end
	end
	if( bbid ~= 0 ) then
		OBB_ChangeTraget(bbid);
	end
end

-------------------------------------------
-- Returns true if it has aggro, else false
-------------------------------------------
function eTargetList.IsAggro(frame)
	local iFrame = _G[frame:GetName().."TGT"];
	if( iFrame:IsVisible() ) then
		return true;
	else
		return false;
	end
end

--========================================================================================================
-- Dropdown config menu
--========================================================================================================
--------------------------
-- Mouse over event(start)
--------------------------
function eTargetList.ConfigButtonOnEnter(this)
	local frame = _G["GameTooltip"];
	frame:ClearAllAnchors();
	frame:SetOwner(this, "ANCHOR_TOP", 0, 0);
	frame:ClearLines();
	frame:SetText(DescConfigButton);
	frame:Show();
end

------------------------
-- Mouse over event(end)
------------------------
function eTargetList.ConfigButtonOnLeave(this)
	_G["GameTooltip"]:Hide();
end

----------------
-- OnClick event
----------------
function eTargetList.ConfigButtonOnClick(this)
	ToggleDropDownMenu(_G["eTL_ConfigMenu"]);
end

---------------
-- OnLoad event
---------------
function eTargetList.DropDownMenuOnLoad(this)
	UIDropDownMenu_Initialize(this, eTargetList_OptionMenu, "MENU");
end

--------------
-- Option menu
--------------
function eTargetList_OptionMenu()
	local old_h = UIDROPDOWNMENU_BUTTON_HEIGHT;		-- hooking global
	UIDROPDOWNMENU_BUTTON_HEIGHT = 18;
	local info = {};
	if( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
		info = {};
		info.text = "Addon Management";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 1;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Bar Fill Order";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 2;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Bar Style";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 3;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Bar Size";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 4;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Scale";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 5;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Label";
		info.hasArrow = 1;
		info.notCheckable = 1;
		info.value = 6;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Reset Config";
		info.notCheckable = 1;
		info.func = function()
			 eTargetList.ResetConfigOnDemand();
		end
		info.value = 7;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Close";
		info.notCheckable = 1;
		info.func = function()
			onConfig = false;
			DCFmsg("eTL:Closing GUI config", 0, 1, 0);
			eTargetList.ConfigChanged();
		end
		info.value = 8;
		UIDropDownMenu_AddButton(info, 1);

		info = {};
		info.text = "Cancel";
		info.notCheckable = 1;
		info.func = function()
		end
		info.value = 9;
		UIDropDownMenu_AddButton(info, 1);
	elseif( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		if( UIDROPDOWNMENU_MENU_VALUE == 1 ) then
			info = {};
			info.text = "Enable";
			info.func = function()
				tempConfig.enable = true;
				eTargetList.ConfigChanged();
			end
			info.checked = tempConfig.enable;
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Disable";
			info.func = function()
				tempConfig.enable = false;
				eTargetList.ConfigChanged();
			end
			info.checked = not tempConfig.enable;
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 2 ) then
			info = {};
			info.text = "TopDown";
			info.func = function()
				tempConfig.fill_order = "topdown";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.fill_order == "topdown" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "BottomUp";
			info.func = function()
				tempConfig.fill_order = "bottomup";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.fill_order == "bottomup" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 3 ) then
			info = {};
			info.text = "Smooth";
			info.func = function()
				tempConfig.bar_tex = "smooth";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.bar_tex == "smooth" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Flat";
			info.func = function()
				tempConfig.bar_tex = "flat";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.bar_tex == "flat" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Banto";
			info.func = function()
				tempConfig.bar_tex = "banto";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.bar_tex == "banto" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 3;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Round";
			info.func = function()
				tempConfig.bar_tex = "round";
				eTargetList.ConfigChanged();
			end
			if( tempConfig.bar_tex == "round" ) then
				info.checked = true;
			else
				info.checked = false;
			end
			info.value = 4;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 4 ) then
			info = {};
			info.text = "Height";
			info.hasArrow = 1;
			info.notCheckable = 1;
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Width";
			info.hasArrow = 1;
			info.notCheckable = 1;
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Space";
			info.hasArrow = 1;
			info.notCheckable = 1;
			info.value = 3;
			UIDropDownMenu_AddButton(info, 2);
		elseif( UIDROPDOWNMENU_MENU_VALUE == 5 ) then
			for i = 1, #scalestep do
				info = {};
				info.text = scalestep[i];
				info.func = function()
					tempConfig.scale = tonumber(scalestep[i]);
					eTargetList.ConfigChanged();
				end
				if( tempConfig.scale == tonumber(scalestep[i]) ) then
					info.checked = true;
				else
					info.checked = false;
				end
				info.value = i;
				UIDropDownMenu_AddButton(info, 2);
			end
		elseif( UIDROPDOWNMENU_MENU_VALUE == 6 ) then
			info = {};
			info.text = "Show LVL";
			info.func = function()
				tempConfig.show_lvl = not tempConfig.show_lvl;
				eTargetList.ConfigChanged();
			end
			info.checked = tempConfig.show_lvl;
			info.value = 1;
			UIDropDownMenu_AddButton(info, 2);

			info = {};
			info.text = "Show HP";
			info.func = function()
				tempConfig.show_hp = not tempConfig.show_hp;
				eTargetList.ConfigChanged();
			end
			info.checked = tempConfig.show_hp;
			info.value = 2;
			UIDropDownMenu_AddButton(info, 2);
		end
	elseif( UIDROPDOWNMENU_MENU_LEVEL == 3 ) then
		_G["DropDownList3"]:SetFrameStrata("TOOLTIP");		-- for fixing sometimes lvl3 menu is hidden by lower lvl ones
		if( UIDROPDOWNMENU_MENU_VALUE == 1 ) then
			for i = 1, #barheightstep do
				info = {};
				info.text = barheightstep[i];
				info.func = function()
					tempConfig.bar_h = tonumber(barheightstep[i]);
					eTargetList.ConfigChanged();
				end
				if( tempConfig.bar_h == tonumber(barheightstep[i]) ) then
					info.checked = true;
				else
					info.checked = false;
				end
				info.value = i;
				UIDropDownMenu_AddButton(info, 3);
			end
		elseif( UIDROPDOWNMENU_MENU_VALUE == 2 ) then
			for i = 1, #barwidthstep do
				info = {};
				info.text = barwidthstep[i];
				info.func = function()
					tempConfig.bar_w = tonumber(barwidthstep[i]);
					eTargetList.ConfigChanged();
				end
				if( tempConfig.bar_w == tonumber(barwidthstep[i]) ) then
					info.checked = true;
				else
					info.checked = false;
				end
				info.value = i;
				UIDropDownMenu_AddButton(info, 3);
			end
		elseif( UIDROPDOWNMENU_MENU_VALUE == 3 ) then
			for i = 1, #barspacestep do
				info = {};
				info.text = barspacestep[i].."%";
				info.isTitle = false;
				info.func = function()
					tempConfig.bar_s = tonumber(barspacestep[i]);
					eTargetList.ConfigChanged();
				end
				if( tempConfig.bar_s == tonumber(barspacestep[i]) ) then
					info.checked = true;
				else
					info.checked = false;
				end
				info.value = i;
				UIDropDownMenu_AddButton(info, 3);
			end
		end
	end
	UIDROPDOWNMENU_BUTTON_HEIGHT = old_h;		-- unhooking global
end

-------------------------------------------
-- Reflect and save config via DropDownMenu
-------------------------------------------
function eTargetList.ConfigChanged()
	eTargetList.InitFrame();
	eTargetList.SaveConfig();
end

-- Onlick event for Addon Manager
function eTargetList.AMOnClick()
	onConfig = true;
	eTargetList.InitFrame();
end

------------------------
-- Slash command handler
------------------------
function eTargetList.CmdHdr(msg)
	local space = msg:find(" ");
	local p = msg:lower(msg);
	local p1, p2;
	if( space ) then
		p1 = p:sub(1, space - 1);
		p2 = p:sub(space + 1, -1):gsub(" ", "");
	else
		p1 = p;
	end
	if( p1 == "" ) then
		if( onConfig ) then
			onConfig = false;
			eTargetList.InitFrame();
			DCFmsg("eTL:Closing GUI config", 0, 1, 0);
		else
			onConfig = true;
			eTargetList.InitFrame();
			DCFmsg("eTL:Opening GUI config", 0, 1, 0);
		end
	elseif( p1 == "reset" ) then
		eTargetList.ResetConfigOnDemand();
		DCFmsg("eTL:config data -> default", 0, 1, 0);
	else
		DCFmsg("invalid parameter(s)", 1, 0.3, 0);
	end
end

------------------------------
-- Resetting config to default
------------------------------
function eTargetList.ResetConfig()
	-- default data -> save data
	for key, value in pairs(defaultConfig) do
		eTL_Config_Save[key] = value;
	end
	eTL_Config_Save.init = true;
	SaveVariables("eTL_Config_Save");
end

----------------------
-- Loading config data
----------------------
function eTargetList.LoadConfig()
	local need2save = false;
	-- saved data -> internal data
	for key, value in pairs(defaultConfig) do
		if( eTL_Config_Save[key] ~= nil ) then
			tempConfig[key] = eTL_Config_Save[key];
		else
			tempConfig[key] = value;		-- if its a new config, use default one
			need2save = true;
		end
	end
	if( need2save ) then
		SaveVariables("eTL_Config_Save");
	end
	if(	not eTL_Config_Save.init ) then
		onConfig = true;					-- open GUI for first boot
		eTL_Config_Save.init = true;
	end
end

---------------------
-- Saving config data
---------------------
function eTargetList.SaveConfig()
	-- internal data -> save data
	for key, value in pairs(tempConfig) do
		eTL_Config_Save[key] = value;
	end
	eTL_Config_Save.init = true;
	SaveVariables("eTL_Config_Save");
end

--------------------------------
-- Resetting config from command
--------------------------------
function eTargetList.ResetConfigOnDemand()
	eTargetList.ResetConfig();		-- default data -> save data
	eTargetList.LoadConfig();		-- saved data -> internal data
	eTargetList.InitFrame();		-- reset frame
end
